@@ -5,6 +5,7 @@ import logging
 import os
 import re
 import sys
+from typing import TYPE_CHECKING
 import webbrowser
 from pathlib import Path
 from shutil import copy
@@ -15,6 +16,9 @@ from urllib.parse import quote
 
 from appdirs import user_config_dir
 
+if TYPE_CHECKING:
+    from .args_controller import Env
+
 # setup logging
 # This cannot be in __init__.py so that we cannot reliably use logger in __init__.py, __main__.py and decorators.py.
 # The reason is __init__.py gets launched when generic Python autocompletion searches for includable modules.
@@ -115,13 +119,12 @@ def get_path(file):
 
 
 class Config:
-    path = get_path("config.ini")
-    cache = {}
+    # path = get_path("config.ini")
+    # cache = {}
 
-    config = configparser.ConfigParser()
-    config.read(path)
-
-    # muted = False  # silence info text
+    # config = configparser.ConfigParser()
+    # config.read(path)
+    config_file: Path | bool | None = None
 
     QUEUED_NAME = ".queues_lines.tmp"
     INVALID_NAME = ".invalidlines.tmp"
@@ -130,106 +133,27 @@ class Config:
     PROJECT_SITE = "https://github.com/CZ-NIC/convey/"
     verbosity: int = logging.INFO  # standard python3 logging level int
 
+    cache_dir: str = ""
+
+    env: "Env"
+
     @staticmethod
     def missing_dependency(library):
         logger.warning(f"Install {library} by `sudo apt install {library}` first or disable `--{library}` in config")
         input("Press any key...")
         raise KeyboardInterrupt
 
-    @staticmethod
-    def integrity_check():
-        from .dialogue import is_yes  # since Config should be the very first package file to be loaded, we postpone this import
-        # Config file integrity check (we may upgrade Convey from but some new parameters needs to be added manually)
-        default_config = configparser.ConfigParser()
-        default_ini = Path(default_path, "config.ini")
-        default_config.read(default_ini)
-        passed = True
-        section_missing = []
-        section_superfluous = []
-        key_missing = {}  # [key] => section
-        key_superfluous = []
-        for section in default_config:
-            missing_section = False
-            if section not in Config.config:
-                print(f"Missing section: {section}")
-                section_missing.append(section)
-                passed = False
-                missing_section = True
-                # continue
-            for key in default_config[section]:
-                if missing_section or key not in Config.config[section]:
-                    print(f"Missing key {key} (defaulting to {repr(default_config[section][key])}) in section: {section}")
-                    key_missing[key] = section
-                    passed = False
-        for section in Config.config:
-            if section not in default_config:
-                print(f"(Config has an unused section: {section})")
-                section_superfluous.append(section)
-                continue
-            for key in Config.config[section]:
-                if key not in default_config[section]:
-                    print(f"Config has an unused key {key} in section: {section}")
-                    key_superfluous.append(key)
-        if not passed:
-            # analyze current config file and get its section start line numbers
-            config_lines = Config.path.read_text().splitlines(keepends=True)
-            section_lines = {}  # [section name] = section start line number
-            for i, line in enumerate(config_lines):
-                try:
-                    section = re.match("\[([^]]*)\]", line).group(1)
-                except AttributeError:
-                    pass
-                else:
-                    section_lines[section] = i
-
-            # analyze the default config file
-            default_lines = default_ini.read_text().splitlines(keepends=True)
-
-            def get_key(iterable, match):
-                """
-                :rtype: yields tuple of (key name, slice start-stop line of the iterable)
-                """
-                comment = None
-                anchors = tuple(iterable)
-                for i, line in enumerate(iterable):  # add missing keys and sections
-                    if line.startswith("#"):
-                        if not comment:
-                            comment = i
-                        continue
-                    if line.startswith(anchors):
-                        key = re.match("[a-zA-Z0-9_]*", line).group(0)
-                        if key in match:  # we have found one of the matching keys
-                            yield key, slice(comment or i, i + 1)
-                    comment = None
-
-            # remove unused keys
-            for key, slice_ in get_key(config_lines, key_superfluous):
-                for i in range(slice_.start, slice_.stop):
-                    config_lines[i] = ""
-            # insert missing keys and sections
-            for key, slice_ in get_key(default_lines, key_missing):
-                block = default_lines[slice_]
-                try:
-                    start_line = section_lines[key_missing[key]]
-                except KeyError:  # insert missing section
-                    section_lines[key_missing[key]] = start_line = len(config_lines)
-                    config_lines.append(f"\n[{key_missing[key]}]")
-                config_lines[start_line] += "\n" + "".join(block)  # insert missing key
-            if is_yes("Should I add the missing keys (and remove the unused) from your config file?"):
-                Config.path.write_text("".join(config_lines))
-                print("Config file has been successfully modified. Restart Convey.")
-            else:
-                print("Please write missing items into the config file before continuing.\nOpening", Config.path, "...")
-                p = Popen(["xdg-open", Config.path], shell=False)
-            exit()
+    # In the past, we had an integrity check here.
+    # Convey checked the integrity of the config.ini to update it with new keys and remove the old ones.
+    # With the mininterface, the user is not obliged anymore to store the configuration in a file as it is more integrated.
 
     @staticmethod
     def init_verbosity(yes=False, verbosity=None, daemon=None):
         # Set up logging and verbosity
         if daemon:
-            Config.set("daemon", True)
+            Config.get_env().process.daemon = True
         if yes:
-            Config.set("yes", True)
+            Config.get_env().cli.yes = True
         if verbosity:
             Config.verbosity = verbosity
         else:
@@ -247,7 +171,10 @@ class Config:
         console_handler.setLevel(Config.verbosity)  # stream handler to debug level
         logging.getLogger().setLevel(min(Config.verbosity, logging.INFO))  # system sensitivity at least at INFO level
 
-        logger.debug("Config file loaded from: {}".format(Config.path))
+        if Config.config_file:
+            logger.debug("Config file loaded from %s", Config.config_file)
+        else:
+            logger.debug("Config file not found at %s", Config.config_file)
 
     @staticmethod
     def get_debugger():
@@ -281,7 +208,7 @@ class Config:
 
     @staticmethod
     def is_debug():
-        return Config.get('debug')
+        return Config.get_env().cli.debug
 
     @staticmethod
     def is_quiet():
@@ -293,48 +220,17 @@ class Config:
 
     @staticmethod
     def is_testing():
-        return Config.get('testing')
+        return Config.get_env().sending.testing
 
     @staticmethod
-    def get(key, section='CONVEY', get=None):
-        """
+    def set_env(env: "Env"):
+        Config.env = env
 
-        :type get: type
-                * if str, return value will always be str (ex: no conversion '1/true/on' to boolean happens)
-                * if list, value tries to be split by a comma
-                * if bool, it tries to convert to True/False from values 0/off/false 1/on/true
-                * if int, you get integer or 0 on error (which means 0 for on/true)
-                * if None
-                    * bool for text 0/off/false 1/on/true
-                    * None for text = '' or non-inserted value
-                    * or any inserted value if non-conforming to previous possibilities
-        """
-        if key not in Config.cache:
-            try:
-                val = Config.config[section][key]
-                if (get is None or get is bool) and val.lower() in BOOLEAN_STATES:
-                    val = BOOLEAN_STATES[val.lower()]
-                elif val == '':
-                    val = get() if get else None
-            except (configparser.Error, KeyError):
-                return get() if get else None
-            Config.cache[key] = val
-        val = Config.cache[key]
-        if get is str and type(val) is not str:
-            try:
-                return str(Config.config[section][key])
-            except KeyError:
-                return ''
-        elif get is list:
-            if val:
-                return [x.strip() for x in val.split(",")]
-            return []
-        elif get is int:
-            try:
-                return int(val)
-            except ValueError:
-                return 0
-        return val
+    @staticmethod
+    def get_env() -> "Env":
+        """ Allow migration from old configparser to the typed Env dataclass """
+        # This is due to migration from static config.ini. In the future, get rid of the static in favour of objects.
+        return Config.env
 
     @staticmethod
     def github_issue(title, body):
@@ -346,17 +242,6 @@ class Config:
         input(f"\nPlease submit a Github issue at {url}"
               "\nTrying to open issue tracker in a browser...")
 
-    @staticmethod
-    def set(key, val, section='CONVEY'):
-        # XX was this needed? if val is None:
-        #    Config.config.remove_option(section, key)
-        # else:
-        Config.cache[key] = val
-        # XX Config.config.set(section, key, str(val))
-
-    cache_dir = ""
-    output = None  # True, False, None or str (path)
-
     @staticmethod
     def set_cache_dir(path):
         Config.cache_dir = path
@@ -379,12 +264,12 @@ def edit(path="config", mode=3, restart_when_done=False, blocking=False):
     """
     if type(mode) is str:  # from CLI
         mode = int(mode)
-    d = {"template": Config.get("mail_template"), "template_abroad": Config.get("mail_template_abroad"),
-         "uwsgi": "uwsgi.ini", "config": "config.ini"}
+    d = {"template": Config.get_env().sending.mail_template, "template_abroad": Config.get_env().sending.mail_template_abroad,
+         "uwsgi": "uwsgi.ini", "config": "convey.yaml"}
     if path in d:
         path = get_path(d[path])
     elif not isinstance(path, Path):
-        input(f"Not found: {path}. Hit Enter...")
+        input(f"Not found: {path}. Use one of {list(d)}. Hit Enter...")
         return
 
     if restart_when_done:
@@ -395,7 +280,8 @@ def edit(path="config", mode=3, restart_when_done=False, blocking=False):
         # we cannot use xdg-open because template.eml would probably launch an e-mail client
         # app = Popen(['xdg-open', path], stdout=PIPE, stderr=PIPE)
         try:
-            editor = run(["xdg-mime", "query", "default", "text/plain"], stdout=PIPE).stdout.split()[0]  # run: blocking, output
+            editor = run(["xdg-mime", "query", "default", "text/plain"],
+                         stdout=PIPE).stdout.split()[0]  # run: blocking, output
             app = Popen(["gtk-launch", editor, path], stdout=PIPE, stderr=PIPE)  # Popen: non blocking
         except FileNotFoundError:
             library = "xdg-utils libgtk-3-bin"
