@@ -16,8 +16,8 @@ import humanize
 from colorama import Fore
 from tabulate import tabulate
 
+from .action import AggregationGroupedRows
 from .config import Config, get_terminal_size
-from .types import Aggregate
 from .whois import Whois
 
 if TYPE_CHECKING:
@@ -29,6 +29,7 @@ class Informer:
 
     def __init__(self, parser: Parser):
         self.parser = parser
+        self.env = parser.env
         self.queue = deque(maxlen=10)
         self.stdout = sys.stdout
         self.stats_stop = Event()
@@ -41,7 +42,7 @@ class Informer:
 
         stdout = StringIO()
 
-        if clear and not Config.get("daemon", get=bool):
+        if clear and not self.env.process.daemon:
             stdout.write("\x1b[2J\x1b[H")
 
         def print_s(s):
@@ -78,9 +79,6 @@ class Informer:
             else:
                 l.append("Aggregate: " + v)
 
-        # XX
-        # if self.parser.redo_invalids is not None:
-        #    l.append("Redo invalids: " + str(self.parser.redo_invalids))
         stdout.write(", ".join(l))
 
         l3 = []
@@ -158,7 +156,7 @@ class Informer:
                 for r in full_rows:
                     cw.writerow(r)
 
-        output = Config.get("output")
+        output = self.env.io.output
         if output:
             print_s(f"Output file specified: {output}")
 
@@ -188,16 +186,18 @@ class Informer:
                     if local[0] + abroad[0] == 0:
                         print_s(f"Already sent all {abroad[1]} abroad e-mails and {local[1]} local e-mails")
                     elif local[1] + abroad[1] > 1:
-                        print_s(f"Already sent {abroad[1]}/{sum(abroad)} abroad e-mails and {local[1]}/{sum(local)} local e-mails")
+                        print_s(
+                            f"Already sent {abroad[1]}/{sum(abroad)} abroad e-mails and {local[1]}/{sum(local)} local e-mails")
                     else:
-                        print_s(f"* {abroad[0]} files seem to be attachments for abroad e-mails\n* {local[0]} for local e-mails")
+                        print_s(
+                            f"* {abroad[0]} files seem to be attachments for abroad e-mails\n* {local[0]} for local e-mails")
                     if non_deliverable:
                         print_s(f"* {non_deliverable} files undeliverable")
 
-                if Config.get('testing'):
+                if self.env.sending.testing:
                     print_s(
                         "\n*** TESTING MOD - mails will be send to mail {} ***\n (For turning off testing mode set `testing = False` in config.ini.)".format(
-                            Config.get('testing_mail')))
+                            self.env.sending.testing_mail))
 
             stat = self.get_stats_phrase()
             if stat:
@@ -231,7 +231,8 @@ class Informer:
                                  {True: "✓", False: "error", None: "no"}[o.sent],
                                  humanize.naturalsize(o.path.stat().st_size),
                                  ))
-                print_s("\n\n** Generated files overview **\n" + tabulate(rows, headers=("file", "deliverable", "sent", "size")))
+                print_s("\n\n** Generated files overview **\n" +
+                        tabulate(rows, headers=("file", "deliverable", "sent", "size")))
             # else:
             #     print_s("Files overview not needed – everything have been processed into a single file.")
 
@@ -243,7 +244,7 @@ class Informer:
             self.stdout.write("".join(self.queue))
             # XX if random true, pops out an element so that it wont stay forever
 
-    def get_aggregation(self, data, color=False, limit=None, nice=True):
+    def get_aggregation(self, data: AggregationGroupedRows, color=False, limit=None, nice=True):
         """
 
         @param data:
@@ -252,7 +253,7 @@ class Informer:
         @param nice: If true, tabulated result returned, else we get (header, rows) tuple.
         @return:
         """
-        form = lambda v, fmt: f"\033[{fmt}m{v}\033[0m" if color else v
+        def form(v, fmt): return f"\033[{fmt}m{v}\033[0m" if color else v
         header = []
         grouping = self.parser.settings["aggregate"].group_by is not None
         if grouping:
@@ -262,23 +263,20 @@ class Informer:
         rows = []
         generators = cycle(g[0] for g in self.parser.settings["aggregate"].actions)
         # sorting by first column (may be a bottleneck, re-sorting every time)
-        dd = sorted(data.items(), key=lambda x: x[1][0][1], reverse=True)
-        for i, (grouped_el, d) in enumerate(dd):
+        for i, (grouped_el, counters) in enumerate(sorted(data.items(), key=lambda x: x[1][0].count, reverse=True)):
             if limit and i == limit:
-                rows.append(["..." for fn, count in d])
+                rows.append(["..." for _ in counters])
                 if grouping:
                     rows[-1].insert(0, "...")
                 break
-            if grouped_el is None and len(self.parser.settings["aggregate"].actions) == 1 \
-                    and next(generators) == Aggregate.list:
+            if grouped_el is None and len(counters) == 1 and counters[0].is_too_broad:
                 # This is the total row
                 # We are aggregating only single thing which is list.
                 # That list would comprehend all of the values in the column. We omit it.
+                rows.append(["(all)"])
                 continue
 
-            # rows.append([form(count if fn.__name__ in ("count", "list") else round(count * 100) / 100, 33) for fn, count in d])
-            rows.append([form(count if next(generators) not in (Aggregate.sum, Aggregate.avg) else round(count, 2), 33) for
-                         fn, count in d])
+            rows.append([form(c.get(), 33) for c in counters])
             if grouping:
                 rows[-1].insert(0, form("total" if grouped_el is None else grouped_el, 36))
         # floatfmt - display numbers longers than 15 as the scientific 1e+15, not numbers bigger than a million only
@@ -326,7 +324,7 @@ class Informer:
                 s = f"({ip_csirtmail_known} IP)"
             if ip_csirtmail_unofficial:
                 s2 = f", no official contact for {abusemail_unofficial} abroad e-mail addresses" \
-                     f" in {csirtmail_unofficial} countries ({ip_csirtmail_unofficial} IP in {prefix_csirtmail_unofficial} prefixes)"
+                    f" in {csirtmail_unofficial} countries ({ip_csirtmail_unofficial} IP in {prefix_csirtmail_unofficial} prefixes)"
             res.append(f"deliverable to {csirtmail_known} national/governmental CSIRTs {s}{s2}")
 
         # interesting if using either incident-contact or abusemail
@@ -429,7 +427,7 @@ class Informer:
             sleep(speed)
 
     def write_statistics(self):
-        if Config.get("write_statistics") and not self.parser.stdin:
+        if self.env.cli.write_statistics and not self.parser.stdin:
             # we write statistics.txt only if we're sourcing from a file, not from stdin
             # XX move this to parser.run_analysis and _resolve_again or to Processor – do not rewrite it every time here!
             stat = self.get_stats_phrase()
