@@ -4,6 +4,7 @@ import smtplib
 import sys
 from abc import abstractmethod, ABC
 from socket import gaierror
+from bs4 import BeautifulSoup
 from requests import get, post
 
 from validate_email import validate_email
@@ -11,9 +12,8 @@ from validate_email import validate_email
 from envelope import Envelope
 
 
-from .dialogue import is_yes
 from .parser import Parser
-from .attachment import Attachment
+from .attachment import Attachment, AttachmentExc
 from .config import Config
 
 re_title = re.compile('<title>([^<]*)</title>')
@@ -21,6 +21,7 @@ logger = logging.getLogger(__name__)
 
 OTRS_VERSION = 6
 
+
 class MailSender(ABC):
     def __init__(self, parser):
         self.parser: Parser = parser
@@ -36,7 +37,7 @@ class MailSender(ABC):
         pass
 
     def send_list(self, mails):
-        """ Send a bunch of e-mail messages 
+        """ Send a bunch of e-mail messages
         :type mails: Union[list, generator]
         """
         sent_mails = 0
@@ -50,10 +51,10 @@ class MailSender(ABC):
 
             try:
                 e: Envelope = attachment.get_envelope()
-            except RuntimeError as e:
+            except AttachmentExc as err:
                 # troubles with an attachment
                 total_count += 1
-                logging.error('Troubles sending the mail for %s. %s', attachment.mail, e)
+                logging.error('Troubles sending the mail for %s. %s', attachment.mail, err)
                 continue
 
             if e.message().strip() == "" or e.subject().strip() == "":
@@ -125,49 +126,73 @@ class MailSenderOtrs(MailSender):
         # context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
         # context.options &= ~ssl.OP_NO_TLSv1
 
-        # if Config.is_debug():
-        # http.client.HTTPConnection.debuglevel = 1
-        # requests_log = logging.getLogger("requests.packages.urllib3")
-        # requests_log.setLevel(logging.INFO)
-        # requests_log.propagate = True
-
         # upload attachments before the main request
         if attachments:
             # get FormID to pair attachments
             logger.debug("Receiving FormID")
-            form_id_r = get(url,
-                            params={"ChallengeToken": self.parser.otrs_token,
-                                    "Action": "AgentTicketForward",
-                                    "TicketID": str(self.parser.otrs_id)},
-                            cookies=cookies
-                            )
-
-            m = re.search(r'FormID" value="((\d|\.)*)"', form_id_r.text)
+            upload_form = get(url,
+                              params={"ChallengeToken": self.parser.sending.otrs_token,
+                                      "Action": "AgentTicketForward",
+                                      "TicketID": str(self.parser.sending.otrs_id)},
+                              cookies=cookies,
+                              verify=self.parser.env.sending.verify_ssl
+                              )
+            m = re.search(r'FormID" value="((\d|\.)*)"', upload_form.text)
             if m:
                 form_id = m[1]
+                self._clean_attachments(upload_form, url, form_id, cookies)
                 for a in attachments:
                     logger.debug("Uploading %s", a.name)
                     post(url,
                          params={"Action": "AjaxAttachment",
                                  "Subaction": "Upload",
                                  "FormID": form_id,
-                                 "ChallengeToken": self.parser.otrs_token
+                                 "ChallengeToken": self.parser.sending.otrs_token
                                  },
                          cookies=cookies,
-                         files={"Files": (a.name, a.data)}
+                         files={"Files": (a.name, a.data)},
+                         verify=self.parser.env.sending.verify_ssl
                          )
 
                 fields["FormID"] = form_id
             else:
                 raise RuntimeError("Cannot get the FormID, unable to upload the attachments.")
 
-        # If we have a single file, we could upload it with single request that way:
-        # `files={"FileUpload": (a.name, a.data)}``
+        # If we had a single file, we could upload it with a single request that way:
+        # `files={"FileUpload": (a.name, a.data)}`
         logger.debug("Submitting mail")
-        r = post(url, params=fields, cookies=cookies)
+        r = post(url, data=fields, cookies=cookies, verify=self.parser.env.sending.verify_ssl)
 
         return r.text
 
+    def _clean_attachments(self, upload_form, url, form_id, cookies):
+        """ Remove all the attachments, received in the first ticket article so that they are not forwareded. """
+        REMOVE_ALL_ATTACHMENTS = True
+        def bs(): return BeautifulSoup(upload_form.text, features="html.parser")
+
+        if REMOVE_ALL_ATTACHMENTS:
+            # why reversed? When deleted, others data-file-id shift down.
+            for td in reversed(bs().find_all('a', {'class': 'AttachmentDelete'})):
+                self._remove_attachment(td['data-file-id'], form_id, url, cookies)
+        elif self.parser.source_file:  # remove at least the attachment we have been split from (avoid duplicity)
+            td = bs().find('td', {'class': 'Filename'}, lambda tag: tag.string == self.parser.source_file.name)
+            if td:  # such attachment exists
+                tr = td.parent
+                data_file_id = tr.find('a', {'class': 'AttachmentDelete'})['data-file-id']
+                self._remove_attachment(data_file_id, form_id, url, cookies)
+
+    def _remove_attachment(self, data_file_id, form_id, url, cookies):
+        logger.debug(f"Removing FileID={data_file_id} from the ticket article attachments")
+        post(url,
+             params={"Action": "AjaxAttachment",
+                     "Subaction": "Delete",
+                     "FormID": form_id,
+                     "ChallengeToken": self.parser.sending.otrs_token,
+                     "FileID": data_file_id
+                     },
+             cookies=cookies,
+             verify=self.parser.env.sending.verify_ssl)
+
     @staticmethod
     def _check_record(record, lineno):
         valid = ('CONTACTS' in record)
@@ -205,38 +230,9 @@ class MailSenderOtrs(MailSender):
             logger.error(response)
         return False
 
-    @staticmethod
-    def ask_value(value, description=""):
-        sys.stdout.write(f'Change {description} ({value})? [s]kip or paste it: ')
-        t = input()
-        if not t or t.lower() == "s":
-            if not value:
-                print("Attention, there should not be an empty value.")
-            return value
-        else:
-            return t
-
     def assure_tokens(self):
         """ Check and update by dialog OTRS credentials """
-        force = False
-        while True:
-            if (force
-                    or not self.parser.otrs_id or not self.parser.otrs_num
-                    or not self.parser.otrs_cookie or not self.parser.otrs_token or not self.parser.attachment_name):
-                self.parser.otrs_id = self.ask_value(self.parser.otrs_id, "ticket url-id")
-                self.parser.otrs_num = self.ask_value(self.parser.otrs_num, "ticket long-num")
-                self.parser.otrs_cookie = self.ask_value(self.parser.otrs_cookie, "cookie")
-                self.parser.otrs_token = self.ask_value(self.parser.otrs_token, "token")
-                self.parser.attachment_name = self.ask_value(self.parser.attachment_name, "attachment name")
-
-            if is_yes(f"Ticket id = {self.parser.otrs_id}, ticket num = {self.parser.otrs_num},"
-                      f" cookie = {self.parser.otrs_cookie}, token = {self.parser.otrs_token},"
-                      f" attachment_name = {self.parser.attachment_name}"
-                      f"\nWas that correct?"):
-                return True
-            else:
-                force = True
-                continue
+        self.parser.m.form(self.parser.sending)
 
     def process(self, e: Envelope):
         def assure_str(c):
@@ -251,7 +247,7 @@ class MailSenderOtrs(MailSender):
         fields = {
             "Action": "AgentTicketForward",
             "Subaction": "SendEmail",
-            "TicketID": str(self.parser.otrs_id),
+            "TicketID": str(self.parser.sending.otrs_id),
             "Email": e.from_().address,
             "From": assure_str(e.from_()),
             "To": assure_str(e.to()),  # mails can be delimited by comma or semicolon
@@ -259,14 +255,14 @@ class MailSenderOtrs(MailSender):
             "Body": message,
             "ArticleTypeID": "1",  # mail-external
             "ComposeStateID": "4",  # open
-            "ChallengeToken": self.parser.otrs_token,
+            "ChallengeToken": self.parser.sending.otrs_token,
         }
 
-        try:  # XX as of Python3.8, convert "try" to if m := Config.get("signkeyid", "OTRS"):
+        try:  # XX as of Python3.8, convert "try" to if m := self.m.env.whois.signkeyid:
             if OTRS_VERSION == 6:
-                fields["EmailSecurityOptions"] = Config.get("signkeyid", "OTRS")
+                fields["EmailSecurityOptions"] = Config.get_env().otrs.signkeyid
             else:
-                fields["SignKeyID"] = Config.get("signkeyid", "OTRS")
+                fields["SignKeyID"] = Config.get_env().otrs.signkeyid
         except KeyError:
             pass
 
@@ -276,7 +272,7 @@ class MailSenderOtrs(MailSender):
             fields["Bcc"] = assure_str(e.bcc())
 
         attachments = e.attachments()
-        cookies = {('OTRSAgentInterface' if OTRS_VERSION == 6 else 'Session'): self.parser.otrs_cookie}
+        cookies = {('OTRSAgentInterface' if OTRS_VERSION == 6 else 'Session'): self.parser.sending.otrs_cookie}
 
         if Config.is_testing():
             print(" **** Testing info:")
@@ -284,8 +280,8 @@ class MailSenderOtrs(MailSender):
             print(' ** Files: ', [(a.name, len(a.data)) for a in attachments])
             print(' ** Cookies: ' + str(cookies))
 
-        host = Config.get("otrs_host", "OTRS")
-        url = (host if "://" in host else f"https://{host}") + Config.get("baseuri", "OTRS")
+        host = Config.get_env().otrs.host
+        url = (host if "://" in host else f"https://{host}") + Config.get_env().otrs.baseuri
         try:
             res = self._post_multipart(url,
                                        fields=fields,
@@ -309,7 +305,7 @@ class MailSenderSmtp(MailSender):
 
     def start(self):
         try:
-            self.smtp = smtplib.SMTP(Config.get("smtp_host", "SMTP"))
+            self.smtp = smtplib.SMTP(Config.get_env().sending.smtp_host)
         except (gaierror, ConnectionRefusedError) as e:
             print("Can't connect to SMTP server", e)
             return False
