@@ -1,5 +1,6 @@
 # Source file parsing
 import csv
+from dataclasses import dataclass
 import datetime
 import logging
 import re
@@ -12,27 +13,41 @@ from math import ceil
 from operator import eq, ne
 from pathlib import Path
 from shutil import move
-from typing import List, Optional, Tuple, Union
+from typing import DefaultDict, List, Optional, Tuple, Union
 from sys import exit
-
 from tabulate import tabulate
 
+from .action import AggregationGroupedRows
+from .args_controller import Env
 from .attachment import Attachment
 from .contacts import Contacts
+from .checker import Checker
 from .config import Config, get_terminal_size
 from .definition import Settings
-from .dialogue import Cancelled, is_yes, ask
+from .dialogue import Cancelled, is_yes
 from .field import Cell, Field
 from .identifier import Identifier
 from .informer import Informer
 from .mail_draft import MailDraft
 from .processor import Processor
-from .types import Types, Type, Web, TypeGroup, Checker
+from .types import Types, Type, TypeGroup
+from .utils import ErrorOnAccess
+from .web import Web
 from .whois import Whois
 
+from mininterface import Mininterface
+
 logger = logging.getLogger(__name__)
 
 
+@dataclass
+class SendingSettings:
+    otrs_id: str = ""
+    otrs_cookie: str = ""
+    otrs_token: str = ""
+    attachment_name: str = "attachment.csv"
+
+
 class Parser:
     is_split: bool
     "has already been split into many files (or we are just about to start file processing)"
@@ -44,7 +59,9 @@ class Parser:
     "has already been processed"
     attachments: List[Attachment]
 
-    def __init__(self, source_file: Path = False, stdin=None, types=None, prepare=True):
+    def __init__(self, m: Mininterface[Env], source_file: Path = False, stdin=None, types=None, prepare=True):
+        self.m = m
+        self.env = m.env
         self.is_formatted = False
         self.is_repeating = False
         self.dialect = None
@@ -68,15 +85,19 @@ class Parser:
         "field types of the columns as given by the user"
         self.settings: Settings = defaultdict(list)
         "processing settings"
-        self.redo_invalids = Config.get("redo_invalids")
+        self.col_seen = set()
+        """ Prevent a column being added multiple time from CLI. """
+
         # OTRS attributes to be linked to CSV
-        self.otrs_cookie = False
-        self.otrs_id = Config.get("ticketid", "OTRS")
-        self.otrs_token = False
-        self.otrs_num = Config.get("ticketnum", "OTRS")
-        self.attachment_name = (source_file.name if source_file else "attachment.csv")
-        if not Path(self.attachment_name).suffix:
-            self.attachment_name += ".csv"
+        default_name = (source_file.name if source_file else "attachment.csv")
+        if not Path(default_name).suffix:
+            default_name += ".csv"
+
+        self.sending = SendingSettings(otrs_id=self.env.otrs.id,
+                                       otrs_cookie=self.env.otrs.cookie,
+                                       otrs_token=self.env.otrs.token,
+                                       attachment_name=default_name)
+
         self.ip_count_guess = None
         self.ip_count = None
         self.attachments = []
@@ -94,9 +115,10 @@ class Parser:
         "CSV processing vs single_query check usage"
         self.ranges = {}  # XX should be refactored as part of Whois
         self.ip_seen = {}  # XX should be refactored as part of Whois
-        self.aggregation = defaultdict(dict)
-        "set by processor [location file][grouped row][order in aggregation settings] = [sum generator, count]"
+        self.aggregation: DefaultDict[str, AggregationGroupedRows] = defaultdict(lambda: defaultdict(list))
+        "set by processor [location file][grouped row] = [AggregationCounter,]"
         self.refresh()
+        self.stats = defaultdict(set)
         self._reset(reset_header=False)
         self.selected: List[int] = []
         "list of selected fields col_i that may be in/excluded and moved in the menu"
@@ -106,12 +128,12 @@ class Parser:
         # load CSV
         self.source_file: Path = source_file or self.invent_file_str()
         self.stdin = []
-        self.stdout = Config.get("stdout")
+        self.stdout = self.env.io.stdout
         """When accepting input from stdin and not saving the output into a file
           or when setting this to True,
           the output will be here.
-        Config.get("output") - the file we should save to
-        Config.get("stdout") - stdout used"""
+        self.env.io.output - the file we should save to
+        self.env.io.stdout - stdout used"""
         self.stdout_sample = None
         self.target_file = None
         self.saved_to_disk = None
@@ -144,6 +166,7 @@ class Parser:
             print("Empty contents.")
             exit()
         self.prepare_target_file()
+        self.sample_parsed = [x for x in csv.reader(self.sample)]
 
         # check if we are parsing a single cell
         if self.stdin:
@@ -176,7 +199,7 @@ class Parser:
             elif not len_ or len_ > 1:
                 seems = False
 
-            if Config.get("single_query") or (seems and Config.get("single_query") is not False):
+            if self.env.io.single_query or (seems and self.env.io.single_query is not False):
                 # identify_fields some basic parameters
                 self.add_field([Field(self.stdin[0])])  # stdin has single field
                 self.dialect = False
@@ -185,27 +208,27 @@ class Parser:
                     # tell the user what type we think their input is
                     # access the detection message for the first (and supposedly only) field
                     detection = self.get_fields_autodetection(False)[0][1]
-                    if not detection and not Config.get("adding-new-fields"):
+
+                    if not detection and not self.env.comp.adding_new_fields:
                         # this is not a single cell despite it was probable, let's continue input parsing
                         logger.info("We could not parse the input text easily.")
                     else:
                         if not detection:
                             # we are adding new fields - there is a reason to continue single processing
                             logger.info("Input value seems to be plaintext.")
-                        elif Config.is_quiet() and Config.get("single_detect", get=bool):
+                        elif Config.is_quiet() and self.env.io.single_detect:
                             print(detection)  # print out detection even if we are quiet because explicitly asked for
                         else:
                             logger.info(f"Input value {detection}\n")
                         self.is_single_query = True
                         return self
-                if Config.get("single_query"):
+                if self.env.io.single_query:
                     logger.info("Forced single processing")
                     self.fields[0].possible_types = {Types.plaintext: 1}
                     self.is_single_query = True
                     return self
 
         # we are parsing a CSV file
-        self.sample_parsed = [x for x in csv.reader(self.sample)]
         self.informer.sout_info()
         try:
             # Dialog to obtain basic information about CSV - delimiter, header
@@ -215,8 +238,8 @@ class Parser:
             l = []
             if self.is_pandoc:
                 l.append("Pandoc table format detected (header were underlined by -------)")
-            if Config.get("delimiter", "CSV"):
-                self.dialect.delimiter = Config.get("delimiter", "CSV")
+            if self.env.csv.delimiter:
+                self.dialect.delimiter = self.env.csv.delimiter
                 l.append(f"Delimiter character set: '{self.dialect.delimiter}'")
                 self.dialect.delimiter = self.dialect.delimiter.replace(r"\t", "\t").replace("TAB", "\t").replace("tab",
                                                                                                                   "\t")
@@ -225,20 +248,20 @@ class Parser:
                 s = "proposed" if seems_single else "found"
                 l.append(f"Delimiter character {s}: '{self.dialect.delimiter}'")
 
-            if Config.get("quote_char", "CSV"):
-                self.dialect.quotechar = Config.get("quote_char", "CSV")
+            if self.env.csv.quote_char:
+                self.dialect.quotechar = self.env.csv.quote_char
                 l.append(f"Quoting character set: '{self.dialect.quotechar}'")
             else:
                 uncertain = True
                 l.append(f"Quoting character: '{self.dialect.quotechar}'")
 
-            if Config.get("header", "CSV") is not None:
-                self.has_header = Config.get("header", "CSV")
+            if self.env.csv.header is not None:
+                self.has_header = self.env.csv.header
             else:
                 uncertain = True
                 l.append(f"Header is present: " + ("yes" if self.has_header else "not used"))
 
-            if Config.get("yes"):
+            if self.m.env.cli.yes:
                 uncertain = False
             else:
                 print("\n".join(l))
@@ -268,17 +291,17 @@ class Parser:
         self.is_formatted = True  # delimiter and header has been detected etc.
         return self
 
-    def get_fields_autodetection(self, append_values=True):
+    def get_fields_autodetection(self, append_values=True) -> list[tuple[Field, str]]:
         """ returns list of tuples [ (field, detection str), ("Url", "url, hostname") ]
         :type append_values: bool Append sample values to the informative result string.
         """
         fields = []
-        for col, field in enumerate(self.fields):
+        for _, field in enumerate(self.fields):
             s = ""
             if field.is_new:
                 s = f"computed from: {field.source_type}"
             elif field.type:
-                s = field.type
+                s = str(field.type)
             elif field.possible_types:
                 types = field.possible_types
                 if Types.any_ip in types and (Types.ip in types or Types.port_ip in types):
@@ -377,12 +400,12 @@ class Parser:
             else:  # loop all existing methods
                 types = Types.get_computable_types(ignore_custom=True)
             for target_type in types:
-                if target_type in Config.get("single_query_ignored_fields", "FIELDS", get=list):
+                if target_type in Config.get_env().comp.single_query_ignored_fields:
                     # do not automatically compute ignored fields
                     continue
                 elif target_type.group == TypeGroup.custom:
                     continue
-                fitting_type = self.identifier.get_fitting_type(0, target_type)
+                fitting_type = self.identifier.get_fitting_type(self.fields[0], target_type)
                 if fitting_type:
                     methods = self.identifier.get_methods_from(target_type, fitting_type, None)
                     if methods:
@@ -416,7 +439,7 @@ class Parser:
                 append(field, val)
 
         # output in text, json or file
-        if Config.get("output"):
+        if self.env.io.output:
             logger.info(f"Writing to {self.target_file}...")
             self.target_file.write_text(dumps(data))
         if json:
@@ -453,11 +476,10 @@ class Parser:
                    unknown_mode=unknown_mode)
 
     def reset_settings(self):
-        self.settings = defaultdict(list)
+        self.settings.clear()
         # when resetting settings, we free up any finished aggregation
         # (because informer wants to display it but the self.parser.settings["aggregate"] is gone
-        self.aggregation = defaultdict(
-            dict)  # self.aggregation[location file][grouped row][order in aggregation settings] = [sum generator, count]
+        self.aggregation.clear()
         self.sample_parsed = [x for x in
                               csv.reader(self.sample[slice(1 if self.has_header else 0, None)],
                                          skipinitialspace=self.is_pandoc,
@@ -473,11 +495,10 @@ class Parser:
         """ Reset variables before new analysis.
         @type reset_header: False if we are in the constructor and added fields is not ready yet.
         """
-        self.stats = defaultdict(set)
+        self.stats.clear()
         Attachment.reset(self.stats)
         self.queued_lines_count = self.invalid_lines_count = self.unknown_lines_count = 0
-        # self.aggregation[location file][grouped row][order in aggregation settings] = [sum generator, count]
-        self.aggregation = defaultdict(dict)
+        self.aggregation.clear()
 
         if reset_header:
             class Wr:  # very ugly way to correctly get the output from csv.writer
@@ -540,7 +561,7 @@ class Parser:
             target_file = f"{'_'.join(l)}{self.source_file.suffix}"
         else:
             target_file = f"output_{time.strftime('%Y-%m-%d %H:%M:%S')}.csv"
-        output = Config.get("output")
+        output = self.env.io.output
         return Path(str(output)) if output else Path(Config.get_cache_dir(), target_file)
 
     def run_analysis(self, autoopen_editor=None):
@@ -550,7 +571,7 @@ class Parser:
         self.refresh()
         self._reset(hard=False)
 
-        if (autoopen_editor or autoopen_editor is None) and Config.get("autoopen_editor") and self.is_split:
+        if (autoopen_editor or autoopen_editor is None) and self.env.cli.autoopen_editor and self.is_split:
             Contacts.mail_draft["local"].edit_text(blocking=False)
             Contacts.mail_draft["abroad"].edit_text(blocking=False)
 
@@ -568,8 +589,8 @@ class Parser:
         if self.unknown_lines_count:
             self.resolve_unknown()
 
-        if self.queued_lines_count and Config.get("lacnic_quota_resolve_immediately", "FIELDS") is not False:
-            self.resolve_queued(Config.get("lacnic_quota_resolve_immediately", "FIELDS"))
+        if self.queued_lines_count and self.env.whois.lacnic_quota_resolve_immediately is not False:
+            self.resolve_queued(self.env.whois.lacnic_quota_resolve_immediately)
 
         self.line_count = 0
 
@@ -655,9 +676,6 @@ class Parser:
         That is the reason we are saying 'up to': "There are ... lines with up to ... IPs".
         """
 
-        # (self.is_split or Config.get("whois_reprocessable_unknown", "FIELDS", get=bool)) \
-        #    and (self.stats["prefix_local_unknown"] or self.stats["prefix_abroad_unknown"]):
-
         if not self.unknown_lines_count:
             # self.stats["ip_local_unknown"] and not self.stats["ip_abroad_unknown"]:
             input("No unknown abusemails. Press Enter to continue...")
@@ -725,7 +743,7 @@ class Parser:
 
     def resolve_invalid(self):
         """ Process all invalid rows. """
-        if Config.get("yes", get=bool) and Config.is_quiet():
+        if self.m.env.cli.yes and Config.is_quiet():
             return False
         if not self.invalid_lines_count:
             input("No invalid rows. Press Enter to continue...")
@@ -744,12 +762,12 @@ class Parser:
                 input("File {} not found, maybe resolving was run in the past and failed. Please rerun again.".format(
                     path))
                 return False
-            if Config.get("yes", get=bool):
+            if self.m.env.cli.yes:
                 res = "n"
             else:
                 s = "Open the file in text editor (o) and make the rows valid, when done, hit y for reanalysing them," \
                     " or hit n for ignoring them. [o]/y/n "
-                res = ask(s)
+                res = self.m.ask(s)
             if res == "n":
                 return False
             elif res == "y":
@@ -821,7 +839,7 @@ class Parser:
         rows = []  # nice table formatting
         full_rows = []  # formatting that optically matches the Sample above
         for line in self.sample_parsed:
-            row: List[Tuple[str,Field]] = []
+            row: List[Tuple[str, Field]] = []
             for field, cell in zip_longest(self.fields, line):
                 if field is None:
                     # when lines have different length, field may be none
@@ -850,7 +868,7 @@ class Parser:
                         unique_sets[col_i].add(row[col_i][0])
 
             # colorize the line
-            g = lambda short: (field.color(cell, short, line_chosen) for cell, field in row)
+            def g(short): return (field.color(cell, short, line_chosen) for cell, field in row)
             rows.append([*g(True)])
             full_rows.append([*g(False)])
         return full_rows, rows
@@ -865,10 +883,20 @@ class Parser:
             fields2 = [fields2]
         return [f1 for f1 in self.fields if f1.type for f2 in fields2 if f1.type == f2.type]
 
-
-
     def __getstate__(self):
         state = self.__dict__.copy()
+        # we get rid of the env and we restore it through .post_setstate.
+        # It might be cool to cache it too, however, we would not be able to distinguish the CLI args.
+        # Usecase:
+        # 1. We have --arg 1 in config defaults
+        # 2. We run with --arg 2
+        # 3. We cache arg=2
+        # 4. We run with --arg 3
+        # There is no easy way to determine, whether to use the config default 1, the cached 2, or the CLI 3.
+        del state['m']
+        del state['env']
+
+        # del other non-cachable objects
         del state['informer']
         del state['processor']
         del state['identifier']
@@ -886,9 +914,14 @@ class Parser:
         return state
 
     def __setstate__(self, state):
+        self.m = ErrorOnAccess()
+        self.env = ErrorOnAccess()
+
         self.__dict__.update(state)
-        self.informer = Informer(self)
-        self.processor = Processor(self, rewrite=False)
+
+        if not isinstance(self.sending, SendingSettings):
+            # Sometimes, we unpickle it as a mere dict. I don't know why.
+            self.sending = SendingSettings(**self.sending)
 
         # input CSV dialect
         self.dialect = csv.unix_dialect()
@@ -902,6 +935,13 @@ class Parser:
             for k, v in d.items():
                 setattr(self.settings["dialect"], k, v)
 
-        self.identifier = Identifier(self)
         self.ranges = {}
         self.ip_seen = {}
+
+    def post_setstate(self, m:Mininterface[Env]):
+        self.m = m
+        self.env = m.env
+
+        self.informer = Informer(self)
+        self.processor = Processor(self, rewrite=False)
+        self.identifier = Identifier(self)
\ No newline at end of file
