@@ -11,13 +11,14 @@ from pathlib import Path
 from queue import Queue, Empty
 from sys import exit
 from threading import Thread, Lock
-from typing import Dict, TYPE_CHECKING, List, Optional, Tuple, Union
+from typing import Dict, TYPE_CHECKING, List, Tuple, Union
 
+from .aggregate import Aggregate
 from .action import Expandable
 from .attachment import Attachment
 from .config import Config
-from .dialogue import ask
-from .types import Web
+from .types import Types
+from .web import Web
 from .whois import Quota, UnknownValue
 
 if TYPE_CHECKING:
@@ -33,8 +34,6 @@ def prod(iterable):  # XX as of Python3.8, replace with math.prod
     return reduce(mul, iterable, 1)
 
 
-
-
 class Processor:
     """ Opens the CSV file and processes the lines. """
 
@@ -73,7 +72,8 @@ class Processor:
         # convert settings["merge"] to lambdas
         if settings["merge"]:
             settings["merging"] = True
-            settings["addByMethod"] += [("merged", op.local_column.col_i_original, (lambda x: op.get(x),)) for op in settings["merge"]]
+            settings["addByMethod"] += [("merged", op.local_column.col_i_original,
+                                         (lambda x: op.get(x),)) for op in settings["merge"]]
             del settings["merge"]
 
         # convert filter settings to pre (before line processing) and post that spare a lot of time
@@ -94,7 +94,8 @@ class Processor:
         if [f for f in self.parser.fields if (not f.is_chosen or f.col_i_original != f.col_i)]:
             settings["chosen_cols"] = [f.col_i_original for f in self.parser.fields if f.is_chosen]
 
-        Web.init([f.type for f in self.parser.get_computed_fields()])
+        used_types = [f.type for f in self.parser.get_computed_fields()]
+        Web.init(Types.text in used_types, Types.html in used_types)
 
         # start file processing
         try:
@@ -117,7 +118,7 @@ class Processor:
 
             # prepare thread processing
             threads = []
-            t = Config.get("threads")
+            t = self.parser.env.process.threads
             if t == "auto":
                 # XX since threads are experimental, make this disable threads
                 # In the future, enable it when using DNS and disable when using WHOIS (that may lead to duplicite calls)
@@ -165,12 +166,12 @@ class Processor:
                     inf.pause()
                     print(f"Keyboard interrupting on line number: {parser.line_count}")
                     s = "[a]utoskip LACNIC encounters" \
-                        if self.parser.queued_lines_count and not Config.get("lacnic_quota_skip_lines", "FIELDS") else ""
-                    o = ask("Keyboard interrupt caught. Options: continue (default, do the line again), "
-                            "[s]kip the line, [d]ebug, [e]nd processing earlier, [q]uit: ")
+                        if self.parser.queued_lines_count and not self.parser.env.whois.lacnic_quota_resolve_immediatel7y else ""
+                    o = self.parser.m.ask("Keyboard interrupt caught. Options: continue (default, do the line again), "
+                                          "[s]kip the line, [d]ebug, [e]nd processing earlier, [q]uit: ")
                     inf.release()
                     if o == "a":
-                        Config.set("lacnic_quota_skip_lines", True, "FIELDS")
+                        self.parser.env.whois.lacnic_quota_skip_lines = True
                     if o == "d":
                         # I dont know why.
                         print("Maybe you should hit n multiple times because pdb takes you to the wrong scope.")
@@ -263,7 +264,7 @@ class Processor:
                 # XX --output [file] does nothing while splitting. In parser.prepare_target_file, we set
                 #   self.is_split = True
                 #   self.target_file = None
-                # We may (a) print out a warning when setting target_file to None if Config.get("output") specified,
+                # We may (a) print out a warning when setting target_file to None if self.env.io.output specified,
                 # (b) set the target_file nevertheless and join the contents from split files here.
                 [[print(x) for x in (f"* Saved to {a.path.name}", "", a.path.read_text())]
                  for a in self.parser.attachments]
@@ -274,7 +275,7 @@ class Processor:
         for f in self.descriptors.values():
             f[0].close()
 
-    def process_line(self, parser: Parser, line: List, settings: Settings, fields: Union[Tuple, List]=None):
+    def process_line(self, parser: Parser, line: List, settings: Settings, fields: Union[Tuple, List] = None):
         """
         Parses line – compute fields while adding, perform filters, pick or delete cols, split and write to a file.
         """
@@ -356,9 +357,10 @@ class Processor:
                 chosen_fields = fields
 
             # determine location
-            if type(settings["split"]) == int:
-                # split ('/' is a forbidden char in linux file names)
-                location = fields[settings["split"]].replace("/", "-")
+            if type(settings["split"]) == int:  # split location:
+                # * '/' is a forbidden char in linux file names)
+                # * we cast to str as the field could contain an object (like IPRange for the 'cidr' type)
+                location = str(fields[settings["split"]]).replace("/", "-")
                 if not location:
                     parser.unknown_lines_count += 1
                     location = Config.UNKNOWN_NAME
@@ -370,26 +372,20 @@ class Processor:
             if settings["aggregate"]:
                 grp = fields[settings["aggregate"].group_by.col_i] if settings["aggregate"].group_by else None
                 for i, (fn, col_data) in enumerate(settings["aggregate"].actions):
-                    # counters[location file][grouped row][order in aggregation settings] = [sum generator, count]
+                    # loc[grouped row] = [Aggregate(sum generator, count),]
                     loc = self.parser.aggregation[location]
 
                     if grp:  # it makes sense to compute total row because we are grouping
-                        if None not in loc:
-                            loc[None] = []
                         if len(loc[None]) <= i:
-                            loc[None].append([fn(), 0])
-                            next(loc[None][i][0])
+                            loc[None].append(Aggregate(fn))
                         if fn.__name__ == "list":
-                            loc[None][i][1] = "(all)"  # we do not want to enlist whole table
+                            loc[None][i].count = "(all)"  # we do not want to enlist whole table
                         else:
-                            loc[None][i][1] = loc[None][i][0].send(fields[col_data.col_i])
+                            loc[None][i].count = loc[None][i].generator.send(fields[col_data.col_i])
 
-                    if grp not in loc:
-                        loc[grp] = []
                     if len(loc[grp]) <= i:
-                        loc[grp].append([fn(), 0])
-                        next(loc[grp][i][0])
-                    loc[grp][i][1] = loc[grp][i][0].send(fields[col_data.col_i])
+                        loc[grp].append(Aggregate(fn))
+                    loc[grp][i].count = loc[grp][i].generator.send(fields[col_data.col_i])
                 return  # we will not write anything right know, aggregation results are not ready yet
         except Quota.QuotaExceeded:
             parser.queued_lines_count += 1
@@ -433,10 +429,8 @@ class Processor:
             if self.descriptors_count >= self.descriptors_max:  # too many descriptors open, we have to close the least used
                 key = min(self.descriptorsStatsOpen, key=self.descriptorsStatsOpen.get)
                 self.descriptors[key][0].close()
-                # print("Closing", key, self.descriptorsStatsOpen[key])
                 del self.descriptorsStatsOpen[key]
                 self.descriptors_count -= 1
-            # print("Opening", location)
 
             if location == 2:  # this is a sign that we store raw data to stdout (not through a CSVWriter)
                 t = w = parser.external_stdout  # custom object simulate CSVWriter - it adopts .writerow and .close methods
