@@ -4,8 +4,10 @@ from collections import OrderedDict
 from datetime import datetime, timedelta
 from subprocess import PIPE, Popen
 from time import time, sleep
+from typing import Literal
 
 from netaddr import IPRange, IPNetwork
+from tldextract import tldextract
 
 from .contacts import Contacts
 from .config import Config, subprocess_env
@@ -13,9 +15,18 @@ from .infodicts import address_country_lowered
 
 logger = logging.getLogger(__name__)
 
-rirs = ["whois.ripe.netf", "whois.arin.net", "whois.lacnic.net", "whois.apnic.net", "whois.afrinic.net"]
 whole_space = IPRange('0.0.0.0', '255.255.255.255')
 
+Ttl = int
+Asn = NetName = Country = str
+Email = str
+CountryMail = Email
+AbuseMail = Email
+IncidentContact = AbuseMail | CountryMail
+Location = Literal["local", "abroad"]
+Prefix = str
+AnalysisResult = tuple[Prefix, Location, IncidentContact, Asn, NetName, Country, AbuseMail, Ttl]
+
 
 class Quota:
     def __init__(self):
@@ -66,46 +77,53 @@ class Whois:
         cls.unknown_mode = unknown_mode  # if True, we use b flag in abusemails
         cls.slow_mode = slow_mode  # due to LACNIC quota
         cls.queued_ips = set()
-        cls.ttl = Config.get("whois_ttl", "FIELDS", int)
+        cls.ttl = Config.get_env().whois.ttl
         cls.see = Config.verbosity <= logging.INFO
-        if Config.get("whois_mirror", "FIELDS"):  # try a fast local whois-mirror first
-            cls.servers["mirror"] = Config.get("whois_mirror", "FIELDS")
+        if mirr := Config.get_env().whois.mirror:  # try a fast local whois-mirror first
+            cls.servers["mirror"] = mirr
         cls.servers["general"] = None
         # Algorithm for querying custom servers:
         # for name, val in zip(["ripe", "arin", "lacnic", "apnic", "afrinic"],
         #                      ["whois.ripe.net -r", "whois.arin.net", "whois.lacnic.net", "whois.apnic.net", "whois.afrinic.net"]):
         #     Whois.servers[name] = val
 
-    def __init__(self, ip):
-        """
-         self.get stores tuple: prefix, location, mail, asn, netname, country, ttl
-        """
+    def __init__(self, ip, hostname=None):
+        """ Access self.get for AnalyzisResult. """
         self.ip = ip
+        self.hostname = hostname
         self.whois_response = []
-        prefix = self.cache_load()  # try load prefix from earlier WHOIS responses
-        if prefix:
-            if (self.ttl != -1 and self.get[7] + self.ttl < time()) or (Whois.unknown_mode and not self.get[6]):
-                # the TTL is too old, we cannot guarantee IP stayed in the same prefix, let's get rid of the old results
-                # OR we are in unknown_mode which means we want abusemail. If not here, maybe another IP claimed
-                # a range superset without abuse e-mail. Delete this possible superset
-                # We do not have to call now `self.get = None; del self.ip_seen[ip]` if there is no need to be thread safe,
-                #   these lines will be called at the function end.
-                del self.ranges[prefix]
+
+        if self.hostname:
+            if len(self.hostname.split(".")) > 2:
+                self.hostname_registerable = self.to_registerable(self.hostname)
             else:
-                self.count_stats()
-                return
+                self.hostname_registerable = self.hostname
+
+        if self.ip:
+            prefix = self.cache_load()  # try load prefix from earlier WHOIS responses
+            if prefix:
+                if (self.ttl != -1 and self.get[7] + self.ttl < time()) or (Whois.unknown_mode and not self.get[6]):
+                    # the TTL is too old, we cannot guarantee IP stayed in the same prefix, let's get rid of the old results
+                    # OR we are in unknown_mode which means we want abusemail. If not here, maybe another IP claimed
+                    # a range superset without abuse e-mail. Delete this possible superset
+                    # We do not have to call now `self.get = None; del self.ip_seen[ip]` if there is no need to be thread safe,
+                    #   these lines will be called at the function end.
+                    del self.ranges[prefix]
+                else:
+                    self.count_stats()
+                    return
 
         if self.see:
-            print(f"Whois {ip}... ", end="", flush=True)
+            print(f"Whois {self.ip or self.hostname_registerable}... ", end="", flush=True)
         if Whois.slow_mode:
             if self.see:
                 print("waiting 7 seconds... ", end="", flush=True)
             sleep(7)
-        get = self.analyze()  # prefix, location, mail, asn, netname, country...
+        get: AnalysisResult = self.analyze()  # AnalyzisResult: prefix, location, mail, asn, netname, country...
         if self.see:
             print(get[2] or "no incident contact.")
         prefix = get[0]
-        if not prefix:
+        if not prefix and self.ip:
             logger.info(f"No prefix found for IP {ip}")
             prefix = IPRange(0, 0)  # make key consistent when saving into cache
         self.ip_seen[ip] = prefix
@@ -156,7 +174,7 @@ class Whois:
                 self.csvstats[f"ip_csirtmail_{known}"].add(self.ip)
                 self.csvstats[f"csirtmail_{known}"].add(country)
 
-        if not mail and Config.get("whois_reprocessable_unknown", "FIELDS", get=bool):
+        if not mail and Config.get_env().whois.reprocessable_unknown:
             raise UnknownValue
 
     def resolve_unknown_mail(self):
@@ -202,14 +220,14 @@ class Whois:
         :param patterns: pattern string or list of strings
         :param last_word: returns only the last word of whole matched expression else last group (ex: the one in parentheses)
 
+        :return:
+        """
         # , take_nth=None, group=None
         # :param take_nth: if available, return n-th result instead of the first available
         #     I.E. `whois 131.72.138.234 | grep ountr` returns three countries: UY, CL, CL.
         #     ARIN registry informs us that this IP is a LACNIC resource and prints out LACNIC address in UY.
         #     However, CL is the country the IP is hosted in.
         # :param group: returned group - default: last group is returned (ex: the one in parentheses)
-        :return:
-        """
         if type(patterns) is str:
             patterns = [patterns]
 
@@ -237,10 +255,7 @@ class Whois:
         #             return line
         # return ""  # no grep result found
 
-    def analyze(self):
-        """
-        :return: prefix, "local"|"abroad", incident-contact ( = abuse-mail|country), asn, netname, country, abuse-mail, TTL
-        """
+    def analyze(self) -> AnalysisResult:
         prefix = country = ""
 
         for server in list(self.servers):
@@ -274,7 +289,7 @@ class Whois:
                         continue
                     if self._match_response("query rate limit exceeded"):  # LACNIC gave me this - seems 300 s needed
                         self.quota.try_start()
-                        if Config.get("lacnic_quota_skip_lines", "FIELDS") and not self.slow_mode:
+                        if Config.get_env().whois.lacnic_quota_skip_lines and not self.slow_mode:
                             if self.see:
                                 print("LACNIC quota exceeded.")
                             self.queued_ips.add(self.ip)
@@ -354,7 +369,7 @@ class Whois:
         if Whois.unknown_mode and not ab:
             ab = self.resolve_unknown_mail()
 
-        local = Config.get("local_country", "FIELDS")
+        local = Config.get_env().whois.local_country
         if local and country not in local:
             mail = Contacts.country2mail[country] if country in Contacts.country2mail else ab
             get1 = "abroad"
@@ -373,27 +388,57 @@ class Whois:
                 return c
         return ""
 
-    reAbuse = re.compile('[a-z0-9._%+-]{1,64}@(?:[a-z0-9-]{1,63}\.){1,125}[a-z]{2,63}')
+    # email regex
+    email_regex = r"[a-z0-9._%+-]{1,64}@(?:[a-z0-9-]{1,63}\.){1,125}[a-z]{2,63}"
+    reAbuse = re.compile(email_regex)
 
     def get_abusemail(self):
-        """ Loads abusemail from last whois response OR from whois json api. """
+        """ Loads abusemail from last whois response """
         match = self.reAbuse.search(self._match_response(['% abuse contact for.*',
                                                           'orgabuseemail.*',
                                                           'abuse-mailbox.*',
-                                                          "e-mail:.*"  # whois 179.50.80.0/21
+                                                          "e-mail:.*",  # whois 179.50.80.0/21,
+                                                          "email:.*"  # ex: 'Registrar Abuse Contact Email: domainabuse@tucows.com',
                                                           ]))
         return match.group(0) if match else ""
 
+    def get_registrar_abusemail(self):
+        """Loads registrar's abusemail from last whois response.
+           Grep a line where both 'abuse', 'registrar' and e-mail are present.
+        """
+
+        abuse_line_pattern = r"^(?=.*\babuse\b)(?=.*\bregistrar\b).*"
+        matches = re.findall(abuse_line_pattern, self.whois_response[0], flags=re.MULTILINE)
+
+        for match in matches:
+            abusemail = re.search(self.email_regex, match)
+            if abusemail:
+                return abusemail.group(0)
+        else:
+            return ""
+
+    def to_registerable(self, url: str) -> str:
+        """Compares the given url with a public list of registerable domains
+
+        ex: website.xyz.com.br -> xyz.com.br
+
+        Note: if a nonregisterable url is given (e.g.: com.br) the same is returned
+        """
+
+        return tldextract.extract(url).registered_domain
+
     regRe = re.compile(r"using server (.*)\.")
 
     def _exec(self, server, server_url=None):
-        """ Query whois server """
+        """Query whois server"""
+        target = self.hostname_registerable if self.hostname else self.ip
+
         if server == "general":
-            cmd = ["whois", "--verbose", self.ip]
+            cmd = ["whois", "--verbose", target]
         else:
             if not server_url:
                 server_url = Whois.servers[server]
-            cmd = ["whois", "--verbose", "-h", server_url, "--", self.ip]
+            cmd = ["whois", "--verbose", "-h", server_url, "--", target]
         self.last_server = None  # check what registry whois asks - may use a strange LIR that returns non-senses
         try:
             # in case wrong env is set to whois, we get `147.32.106.205` country NL and not CZ
@@ -404,7 +449,7 @@ class Whois:
         except UnicodeDecodeError:
             # ip address 94.230.155.109 had this string 'Jan Krivsky Hl\xc3\x83\x83\xc3\x82\xc2\xa1dkov' and everything failed
             self.whois_response = []
-            logger.warning("Whois response for IP {} on server {} cannot be parsed.".format(self.ip, server))
+            logger.warning("Whois response for IP {} on server {} cannot be parsed.".format(target, server))
         except TypeError:  # could not resolve host
             self.whois_response = []
         except FileNotFoundError:
