@@ -15,11 +15,15 @@ from time import time
 
 import ezodf
 import jsonpickle
+from mininterface import Mininterface
+from mininterface.tag import PathTag
 import openpyxl
 import xlrd
 from netaddr import IPRange
 from xlrd import XLRDError
 
+from .args_controller import Env
+
 from .config import Config, config_dir
 from .dialogue import hit_any_key, is_yes
 from .identifier import Identifier
@@ -34,45 +38,45 @@ __date__ = "$Mar 23, 2015 8:33:24 PM$"
 WHOIS_CACHE = ".convey-whois-cache.json"
 
 
-def choose_file():
-    print("Set path to the source log file.")
-    print("? ", end="")
-
-    # XX in the future, let's get rid of Tkinter. And don't impose it now if not really needed
-    try:
-        import tkinter as tk
-        from tkinter.filedialog import askopenfilename
-        root = tk.Tk()
-        root.withdraw()  # show askopenfilename dialog without the Tkinter window
-        file = askopenfilename()  # default is all file types
-        print(file)
-        return file
-    except ImportError:
-        print("Error importing Tkinter. Please specify the file name in the parameter or install `apt install python3-tk`.")
-        return None
+def choose_file(m: Mininterface):
+    # NOTE it would be nice to directly open the file picker
+    return m.form({"Source file": PathTag(description="Set path to the source log file.")})["Source file"]
 
 
 def read_stdin():
-    if Config.get("daemon", get=bool):
+    if Config.get_env().process.daemon:
         raise ConnectionRefusedError("STDIN missing")
     print("Write something to stdin. (End of transmission 2× <Ctrl>+d or <Enter>+<Ctrl>+d.)")
     return sys.stdin.read().rstrip().split("\n")  # rstrip \n at the end of the input
 
 
 class Wrapper:
-    def __init__(self, file_or_input, force_file=False, force_input=False,
+    def __init__(self, m: Mininterface[Env], file_or_input, file_given=False, input_given=False,
                  types=None, fresh=False, reprocess=False, delete_cache=False):
         if delete_cache and Path(config_dir, WHOIS_CACHE).exists():
             Path(config_dir, WHOIS_CACHE).unlink()
 
+        self.m = m
+        self.env = m.env
+        self.whois = m.env.whois
+
         self.parser: Parser = None
         self.file = file = None
         self.stdin = stdin = None
         self.types = types
         self.whois_not_loaded = fresh
         self.last_hash = None
+
+        force_file = force_input = False
+        if file_given:
+            force_file = True
+            file_or_input = file_given
+        elif input_given:
+            force_input = True
+            file_or_input = input_given
+
         try:
-            case = int(Config.get("file_or_input"))
+            case = int(self.env.io.default_action)
         except (ValueError, TypeError):
             case = 0
 
@@ -84,7 +88,7 @@ class Wrapper:
         if force_input:
             stdin = file_or_input.split("\n") if file_or_input else read_stdin()
         elif force_file:
-            file = file_or_input if file_or_input else choose_file()
+            file = file_or_input if file_or_input else choose_file(self.m)
         elif isinstance(file_or_input, Path) or (file_or_input and len(file_or_input) < 256 and Path(file_or_input).is_file()):
             # if longer than 255, it is most probably not a file but input
             file = file_or_input
@@ -92,17 +96,18 @@ class Wrapper:
             stdin = file_or_input.split("\n")
         else:  # choosing the file or input text
             if case == 0:
-                case = 1 if is_yes("Do you want to input text (otherwise you'll be asked to choose a file name)?") else 2
+                case = 1 if is_yes(
+                    "Do you want to input text (otherwise you'll be asked to choose a file name)?") else 2
             if case == 1:
                 stdin = read_stdin()
             elif case == 2:
-                file = choose_file()
+                file = choose_file(self.m)
             elif case == 3:
                 stdin = read_stdin()
                 if not stdin:
-                    file = choose_file()
+                    file = choose_file(self.m)
             elif case == 4:
-                file = choose_file()
+                file = choose_file(self.m)
                 if not file:
                     stdin = read_stdin()
 
@@ -113,7 +118,7 @@ class Wrapper:
             Config.set_cache_dir(Path.cwd())
             self.cache_file = None
             self.stdin = stdin
-            self.parser: Parser = Parser(stdin=stdin, types=self.types)
+            self.parser: Parser = Parser(self.m, stdin=stdin, types=self.types)
             return
 
         if not Path(file).is_file():
@@ -125,6 +130,7 @@ class Wrapper:
             logger.info(f"File {self.file} has already been processed.")
             try:  # try to depickle
                 self.parser = jsonpickle.decode(self.cache_file.read_text(), keys=True)
+                self.parser.post_setstate(self.m)
             except:
                 print(traceback.format_exc())
                 if not Config.error_caught():
@@ -178,14 +184,14 @@ class Wrapper:
     def load_whois_cache(self):
         """ restore whois cache and remove expired results """
         p = Path(config_dir, WHOIS_CACHE)
-        if Config.get("whois_cache", "FIELDS", get=bool) and p.exists():
+        if self.whois.cache and p.exists():
             # XX if it's long, postpone via a thread that would block analysis
             event = lazy_print("... loading big WHOIS cache ...")
             ip_seen, ranges = jsonpickle.decode(p.read_text(), keys=True)
             ranges = {IPRange(k[0], k[1]): v for k, v in ranges.items()
-                      if v[7] + Config.get("whois_ttl", "FIELDS", int) >= time()}
+                      if v[7] + self.whois.ttl >= time()}
             nothing_to_save = True
-            if Config.get("whois_delete_unknown", get=bool) and IPRange(0, 0) in ranges:
+            if self.whois.delete_unknown and IPRange(0, 0) in ranges:
                 # all IP addresses within an unknown prefix removed from cache
                 del ranges[IPRange(0, 0)]
                 nothing_to_save = False
@@ -219,15 +225,15 @@ class Wrapper:
             target_file = self.parser.target_file or self.parser.source_file
             if not target_file:  # we were splitting so no target file exist
                 target_file = self.parser.invent_file_str()
-            if Config.get("output") is None:
-                i = Config.get("save_stdin_output", get=int)
+            if self.env.io.output is None:
+                i = self.env.io.save_stdin_output
                 save = False
                 if i == 4 or (i == 3 and self.parser.is_analyzed):
                     save = True
                 elif i == 2 or (i == 1 and self.parser.is_analyzed) and last_chance:
                     save = is_yes(f"Save to an output file {target_file}?")
             else:
-                save = bool(Config.get("output"))
+                save = bool(self.env.io.output)
             if save:
                 if not self.parser.target_file:
                     # if a split files were generated, we lost the connection because cache dir will change right now
@@ -256,15 +262,15 @@ class Wrapper:
             # Unfortunately, I have not been able to simulate this behaviour with another simpler object than self.parser
             # so that I could raise an official issue.
             jsonpickle.decode(string, keys=True)
-        except Exception:
+        except Exception as e:
             hit_any_key("The program state is not picklable by 'jsonpickle' module. "
-                  "Continuing will provide a file that will have to be reanalysed. "
-                  "You may post this as a bug to the project issue tracker.")
+                        "Continuing will provide a file that will have to be reanalysed. "
+                        "You may post this as a bug to the project issue tracker.")
             Config.error_caught()
 
         # save cache file
         if self.cache_file:  # cache_file does not exist typically if reading from STDIN
-            if self.parser.ranges and Config.get("whois_cache", "FIELDS", get=bool):
+            if self.parser.ranges and self.env.whois.cache:
                 # we extract whois info from self.parser and save it apart for every convey instance
                 if self.whois_not_loaded:  # if we wanted a fresh result, global whois cache was not used and we have to merge it
                     ip_seen, ranges = self.load_whois_cache()
@@ -287,8 +293,8 @@ class Wrapper:
                     except Exception:  # again, I met a strangely formed JSON
                         type_, value, tb = sys.exc_info()
                         body = f"```bash\n{traceback.format_exc()}```\n\n" \
-                               f"```json5\n{tb.tb_next.tb_frame.f_locals}\n```\n\n" \
-                               f"```json5\n{ip_seen}```\n\n```json5\n{ranges}```"
+                            f"```json5\n{tb.tb_next.tb_frame.f_locals}\n```\n\n" \
+                            f"```json5\n{ip_seen}```\n\n```json5\n{ranges}```"
                         print("The program will recover but without saving WHOIS cache.")
                         Config.github_issue("Cannot jsonpickle whois", body)
                     else:
@@ -300,14 +306,14 @@ class Wrapper:
     def clear(self):
         self.check_ods() or self.check_xlsx() or self.check_xls() or self.check_log()
 
-        self.parser = Parser(self.file, self.stdin, self.types)
+        self.parser = Parser(self.m, self.file, self.stdin, self.types)
         self.save()
 
     def check_log(self):
         """ Check if the contents is a CSV and not just a log
          ex: "06:25:13.378767 IP 142.234.39.36.51354 > 195.250.148.86.80: Flags [S], seq 1852455482, win 29200, length 0"
          """
-        re_ip_with_port = re.compile("((\d{1,3}\.){4})(\d+)")
+        re_ip_with_port = re.compile(r"((\d{1,3}\.){4})(\d+)")
         re_log_line = re.compile(r"([^\s]*)\sIP\s([^\s]*)\s>\s([^\s:]*)")
         _, sample, _ = Identifier(None).get_sample(self.file)
         if sample and re_log_line.match(sample[0]):
@@ -381,7 +387,6 @@ class Wrapper:
                         target.writerow(row)
             return True
 
-
     @contextmanager
     def rework(self):
         target = Path(str(self.file) + ".csv")
